/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalLine.h"
#include "HvControlVar.h"
#include "HvControlCast.h"
#include "HvMath.h"
#include "HvSignalVar.h"
#include "HvControlBinop.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_m7FTRGXE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_f71Xno6A_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_iANBI0KR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_TILJy7Vg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_UkWnV1tZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_LQWHvOU5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Y3ffmLBU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_fQ3vZQ7P_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_JpJVt3XD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_OOcrylxO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_HtOKFmMH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_29jF09vT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_VZH6IMyd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_tYjxyDyD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_l65i9svQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_ZgMuplUp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_vRSUJQoi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RYVk7zJt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_5Sslg9hS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_sN3JwnEI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_lOAdB751_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_UrsYOPl6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_MnX8d406_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Epu4Aso4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_pv0hT0Sk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_pqy9M2gs_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_IP9dSaQS;
  SignalLine sLine_aRI8fuhX;
  SignalLine sLine_OwHvoGIp;
  ControlBinop cBinop_m7FTRGXE;
  ControlBinop cBinop_f71Xno6A;
  ControlBinop cBinop_iANBI0KR;
  ControlBinop cBinop_TILJy7Vg;
  ControlBinop cBinop_Y3ffmLBU;
  ControlBinop cBinop_JpJVt3XD;
  ControlBinop cBinop_OOcrylxO;
  ControlBinop cBinop_HtOKFmMH;
  ControlVar cVar_29jF09vT;
  ControlVar cVar_VZH6IMyd;
  SignalVarf sVarf_lbSXUUQo;
  SignalVarf sVarf_RSsAFSmJ;
  SignalVarf sVarf_8rDQBznU;
  SignalVarf sVarf_N27dPSjZ;
  ControlBinop cBinop_RYVk7zJt;
  ControlBinop cBinop_UrsYOPl6;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
